﻿using System.Linq;
using System.Threading;
using System.Threading.Tasks;
using InternshipSystem.Api.Commands;
using InternshipSystem.Api.Security;
using InternshipSystem.Api.UseCases;
using InternshipSystem.Repository;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Http;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;

namespace InternshipSystem.Api.Controllers
{
    [Route("internshipRegistration")]
    public class InternshipRegistrationController : ControllerBase
    {
        private readonly InternshipDbContext _dbContext;

        public InternshipRegistrationController(InternshipDbContext dbContext)
        {
            _dbContext = dbContext;
        }
        
        /// <summary>
        /// Validate and add filled internship registration form 
        /// </summary>
        /// <response code="200">If registration form was successfully added</response>
        /// <response code="400">If the provided registration query was malformed</response>
        /// <response code="401">This action is only available for authorized student registered for current edition</response>
        [HttpPut]
        [ProducesResponseType(StatusCodes.Status200OK)]
        [ProducesResponseType(StatusCodes.Status400BadRequest)]
        [ProducesResponseType(StatusCodes.Status401Unauthorized)]
        [Authorize(Policy = Policies.RegisteredOnly)]
        public async Task<ActionResult> SubmitRegistrationForm(
            UpdateRegistrationForm registrationCommand, 
            [FromServices] User user, 
            CancellationToken cancellationToken)
        {
            var edition = await _dbContext.Editions.FirstAsync(e => e.Id == user.EditionId, cancellationToken);

            var internshipRegistration =
                await _dbContext
                    .Entry(edition)
                    .Collection(e => e.Internships)
                    .Query()
                    .Where(i => i.Student.Id == user.PersonNumber)
                    .Select(i => i.InternshipRegistration)
                        .Include(r => r.BranchAddress)
                        .Include(r => r.Company)
                        .ThenInclude(c => c.Branches)
                    .FirstAsync(cancellationToken);
            
            var useCase = new UpdateInternshipRegistrationUseCase(_dbContext, internshipRegistration, edition, user);
            
            var result = await useCase.UpdateInternshipRegistration(registrationCommand, cancellationToken);
            
            await _dbContext.SaveChangesAsync(cancellationToken);

            return Ok(result);
        }
    }
}